/* A first example of "relational programming": list reversal.
   rev(As, Bs)  holds true if  Bs  is a list holding exactly
   As's  elements but in reverse order. 
        TRY IT:  rev([a,b,c], Ans).
                 rev(Arg, [1,2,3]).
                 rev([0|Stuff], Ans).
                 rev(Any1, Any2).
*/
rev([], []).
rev([X|Xs], Zs) :-  append(Ys, [X], Zs),  rev(Xs, Ys).



/* double(L, M)  holds true if  L  is a list of ints 
   and  M  is a list of ints whose values are exactly 
   those of  L  multiplied by 2.  
        TRY IT:  trace(double).
                 double([1,3,5], A).
*/
double([], []).
double([H|T], [HH|TT]) :-  HH is 2 * H,  double(T, TT).



/* insert(X, L, LwithX)  inserts element X into list L,
   generating  LwithX 
        TRY IT:  trace(insert).
                 insert(4, [1,3,5], A).
*/
insert(X, [], [X]).
insert(X, [Y|Ys], [X, Y | Ys]) :-  X =< Y.
insert(X, [Y|Ys], [Y|Zs]) :-  X > Y,  insert(X, Ys, Zs).


/* isort(Xs, Ys)  generates  Ys  so that it is 
   the sorted  variant of Xs 
*/
isort([], []).
isort([X|Xs], Ys) :-  isort(Xs, Zs), insert(X, Zs, Ys).




/* "relational programming" lets you choose which argument
   slots are "in-params" and "out-params":  */

likes(john, Y) :- sweet(Y).
likes(X, kim).
likes(ed, Y).
sweet(icecream).
sweet(mary).

/* possible queries:  what does john like?
                      who likes john?
                      does mary like john?
                      who likes what?
*/




/* append(L1, L2, M)  holds true if list M  is exactly 
   list L1 appended to list L2  */
append([], L2, L2).
append([H|T], L2, [H|A]) :- append(T, L2, A).


/* You can run this one "backwards" and "forwards" since
   there is no arithmetic inside it:
       TRY IT:   append([1,2], [a,b], Ans).
                 append(X, [1,2], [0,1,2]).
                 append(X, Y, [0,1,2]).
                 append(X, Y, [0|Z]).
*/